#!/usr/bin/env python3

"""Stub setup.py for use with legacy build tooling."""

# Future imports
from __future__ import (
    annotations,
)

# Third party imports
import setuptools

if __name__ == "__main__":
    setuptools.setup()
